#pragma once

#include "lvlwrap.h"
#include "miscstructs.h"
#include "packethelper.h"
#include "waypoint.h"

#define FOLLOW_MODULE_NAME	"follow"
#define VERSION_MAJOR		0
#define VERSION_MINOR		3

#define DEBUG_OFF			0x00
#define DEBUG_USER			0x01
#define	DEBUG_DEVEL			0x02
#define DEBUG_PACKET		0x03

#define LEN_CHARNAME		16	// 16 byte string for character name

// BOT STATES
#define STATE_GOTO_TOWN_IDLE 0
#define STATE_GOTO_TOWN_SWITCHING_SKILL 1
#define STATE_GOTO_TOWN_USING_SKILL 2

// bot attacking skill
#define STATE_SKILL_LEFT		0
#define STATE_SKILL_RIGHT		1
#define STATE_SKILL_SHIFT_LEFT	2
#define STATE_SKILL_SHIFT_RIGHT	3

// bot attacking states
#define STATE_ATK_NORMAL		0
#define STATE_ATK_AGGRESSIVE    1
#define STATE_ATK_DEFENSIVE     2
#define STATE_ATK_TANK          4
#define STATE_ATK_EVADE         8

#define AGGRESSIVE	"AGG"
#define DEFENSIVE	"ON"
#define NORMAL		"OFF"

// (default) chat msg text command from the leader
#define LEADERCMD_GOFIELD "go"
#define LEADERCMD_RETTOWN "town"
#define LEADERCMD_STAY    "stay"
#define LEADERCMD_FOLLOW  "follow"
#define LEADERCMD_ATTACK  "attack"
#define LEADERCMD_TAKETP  "take tp"
#define LEADERCMD_SWITCH  "switch"
#define LEADERCMD_WAYPOINT  "wp"
#define LEADERCMD_WEAPONSWITCH  "w"
#define LEADERCMD_GETHEAL	"heal"
// array size constants
#define MAX_PLAYER_IN_GAME 8
#define TP_ARRAY_SIZE	16
#define LVL_ARRAY_SIZE	64
#define HEALER_ARRAY_SIZE 5

#define ON	"ON"
#define OFF "OFF"

// ini sections
#define INI_SECT_STATE	"BotState"
#define INI_SECT_CMD	"Commands"
// ini strings
#define INI_ATTK_STATE  "ATTACK"
#define INI_SYNC_STATE  "SYNCBLOCK"
#define INI_SHIFT_STATE "SHIFTATTACK"
#define INI_AUTO_ATTK	"AUTOATTACK"
#define INI_OVERHEAD_MSG "NOTIFY"
#define INI_TELE_STATE	"TELEPORT"

#define INI_GO		"CMD_GOFIELD"
#define INI_TOWN	"CMD_RETTOWN"
#define INI_STAY	"CMD_STAY"
#define INI_FOLLOW	"CMD_FOLLOW"
#define INI_ATTK	"CMD_ATTACK"
#define INI_TP		"CMD_TAKETP"
#define INI_SWITCH	"CMD_SWITCH"
#define INI_WP		"CMD_WAYPOINT"
#define INI_WS		"CMD_WEAPONSWITCH"
#define INI_HEAL	"CMD_GETHEAL"
//------------------------------------------------//
// TYPEDEF
//------------------------------------------------//

typedef struct packetcommandstruct_t
{
	byte	bPacket;
    BOOL    isSendPacket; // true for send, false for recv
	DWORD	(PRIVATE *pFunc)(BYTE* aPacket, DWORD aLen);
	char*	szPacketInfo;
} PACKETCOMMANDSTRUCT;


typedef struct playeringame_t
{
	bool		InUse;
	DWORD		PlayerID;	
	char		PlayerName[16];
} PLAYERINGAME, *PPLAYERINGAME;

typedef struct tp_info_t
{
	bool inTown;
	char playerName[16]; // owner of portal
	BYTE portalPos[4];
	BYTE portalIdA[4];		
	BYTE portalIdB[4];		
	BYTE extrInfo[2]; // (0 - ?) (1 - to location)
} PORTALINFO;

typedef struct lvl_wrap_t
{
	DWORD	objId;	
	BYTE	lvl;
	WORD	x;
	WORD	y;
} LVL_WRAP_INFO;

typedef struct skill_t {
	char			name[64];
	WORD			skillID;
	char			d2class[5];
	bool			is_aura;
	int				cast_delay;
	bool			no_left_skill; // 0 - either side, 1 - right only
	bool			is_castable_intown;
	bool			is_passive;
} SKILLSTRUCT;

typedef struct npc_healer_t {
	char			name[32];
	WORD			baseID;
	WORD			entityID;
} NPCHEALERSTRUCT;


typedef struct gateway_t {
	DWORD			entityID;
	//BOOL			isStale;
	//BOOL			isTown;
	char			name1[64];
	LOCATIONSTRUCT	pos;
	WORD			lid;
	//WORD			wpNum;
	char			name2[64];
} GATEWAYSTRUCT;

#define GATEWAY_COUNT 8

//------------------------------------------------//
// Variables
//------------------------------------------------//
// global vars
PACKETCOMMANDSTRUCT PacketCmd[];
extern WORD Debug;
extern char DebugBuffer[1024];
extern SKILLSTRUCT game_skills[];
extern NPCHEALERSTRUCT healer[];
extern GATEWAYSTRUCT last_gw, arrPortals[], gateway[];
extern int arrP_count;
extern char	*cmd_gofield,		// for user defined values in ini
		*cmd_rettown,		*cmd_stay,			*cmd_heal,
		*cmd_follow,		*cmd_attack,
		*cmd_taketp,		*cmd_switch,
		*cmd_waypoint,		*cmd_weaponswitch;

extern PLAYERINGAME playerInGame[MAX_PLAYER_IN_GAME];	// store player name & id in current game
extern LVL_WRAP_INFO lvl_entrance[LVL_ARRAY_SIZE];      // store location & id of cave/lvl entrances
extern PORTALINFO portals[TP_ARRAY_SIZE];		        // info about town portal from leader
extern WPSTRUCT	wp;						// store the last wp encountered
extern THISGAMESTRUCT *thisgame;		// pointer to current game
extern LOCATIONSTRUCT leaderLoc;		// location of leader
										// TODO: below to be moved into portals[]
//extern BYTE portalPosInTown[4];			// my tp pos in town, for returning to field
extern BYTE portalPos[4];				// my tp pos in field
extern BYTE saPacket[9];				// go in TP packet

extern DWORD stateGotoTown;				// current state of follower
extern bool toFollow;					// flag to follow other play or not
extern bool toAttack;					// flag to attack monster or not
extern bool blockSync;					// flag to block sync msg
extern bool autoDrink;					// flag to auto drink potion or not
extern bool autoBuy;					// flag to auto buy potions/arrows/etc
extern bool autoRepair;					// flag to auto repair all
extern bool autoResurr;					// flag to auto resurr after death
extern bool justDied;					// flag to suspend action when bot just died
extern bool toNotify;					// flag to notify master via overhead msg
extern bool toTeleport;					// flag to teleport with master or not
extern bool shiftState;					// shift attack or not
extern int attackSkill;					// flag to control which skill to use for attack
extern int autoAttack;					// flag to control auto attack monster
extern int StateInParty;				// party flag
extern DWORD leaderId;					// ID of the player to follow
extern int leaderPos;					// index of player to follow in playerInGame array
extern char leaderName[16];				// string contains master name (for ini or command start)
extern int CurrentHP, CurrentMP;		// HP, MP monitor
extern int lastHealer_seen;				// index to the last heal bot sees, 0 - act1, 1 - act 2, etc
extern int portal_count;

//------------------------------------------------//
// User Commands
//------------------------------------------------//
BOOL PRIVATE OnCommandDebug(char **argv, int argc);
BOOL PRIVATE OnGameCommandAuto(char** argv, int argc);
BOOL PRIVATE OnGameCommandList(char** argv, int argc);
BOOL PRIVATE OnGameCommandSave(char** argv, int argc);
BOOL PRIVATE OnGameCommandStop(char** argv, int argc);
BOOL PRIVATE OnGameCommandBlock(char** argv, int argc);
BOOL PRIVATE OnGameCommandShift(char** argv, int argc);
BOOL PRIVATE OnGameCommandStart(char** argv, int argc);
BOOL PRIVATE OnGameCommandAttack(char** argv, int argc);
BOOL PRIVATE OnGameCommandStatus(char** argv, int argc);
BOOL PRIVATE OnGameCommandReload(char** argv, int argc);
BOOL PRIVATE OnGameCommandNotify(char** argv, int argc);

//------------------------------------------------//
// internal functions
//------------------------------------------------//

DWORD GetBitField(BYTE* data, DWORD pos, DWORD len, DWORD max);

void readInitFile();
void set_setting(char* sect, char* key, bool val);
char* get_setting(char* sect, char* str, char * default_setting);

int addPlayerInfo(BYTE* aPacket, DWORD aLen);
int add_PlayerInfo(BYTE* playerName, BYTE playerID);
int removePlayerInfo(BYTE* aPacket, DWORD aLen);

void setSkill_TP();
void setRightSkill(WORD skill);
void castTownPortal(BYTE* aPacket, DWORD aLen);
void walkInPortal(BYTE* aPacket, DWORD aLen);
int get_tp_by_name(char* ownerName);
bool take_tp_by_name(char* ownerName);

void followLeader(BYTE* aPacket, DWORD aLen);
void run_to_pos(DWORD *xy);
void teleport_to_pos(DWORD *xy);

void checkLeaderCmd(BYTE* aPacket, DWORD aLen);

BOOL isGateWay(WORD lid);
void print_Packet(BYTE* aPacket, DWORD aLen);

DWORD WINAPI SleepThread(LPVOID lpParameter);
DWORD WINAPI ResetThread(LPVOID lpParameter);
DWORD WINAPI ResurrThread(LPVOID lpParameter);
void OverheadMsg(char *);
void ChatMsg(char *);
int get_healer_by_name(char * npcName);
//------------------------------------------------//
// packet functions - send packet commands
//------------------------------------------------//
DWORD client_walk_update(BYTE* aPacket, DWORD aLen);
DWORD deny_client_sync(BYTE* aPacket, DWORD aLen);

//------------------------------------------------//
// packet functions - receive packet commands
//------------------------------------------------//
DWORD posUpdate(BYTE* aPacket, DWORD aLen);
DWORD player_movement(BYTE* aPacket, DWORD aLen);
DWORD player_nontarget_attack(BYTE* aPacket, DWORD aLen);
DWORD player_target_attack(BYTE* aPacket, DWORD aLen);
DWORD cave_notification(BYTE* aPacket, DWORD aLen);
DWORD cave_follow(BYTE* aPacket, DWORD aLen);
DWORD incoming_chat_msg(BYTE* aPacket, DWORD aLen);
DWORD player_notification(BYTE* aPacket, DWORD aLen);
DWORD new_player(BYTE* aPacket, DWORD aLen);
DWORD townportal_info(BYTE* aPacket, DWORD aLen);
DWORD monster_moving(BYTE* aPacket, DWORD aLen);
DWORD monster_approaching(BYTE* aPacket, DWORD aLen);
DWORD player_info(BYTE* aPacket, DWORD aLen);
DWORD skill_selected(BYTE* aPacket, DWORD aLen);
DWORD player_join(BYTE* aPacket, DWORD aLen);
DWORD healer_info(BYTE* aPacket, DWORD aLen);


